#!/usr/bin/python
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#   Light Locker Settings - simple configuration tool for light-locker
#   Copyright (C) 2014 Thomas Molloy <beetyrootey@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify it
#   under the terms of the GNU General Public License version 3, as published
#   by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranties of
#   MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import argparse
import shlex
import os
import subprocess
from gi.repository import Gtk

''' Settings window for the light-locker '''


class LightLockerSettings:
    '''Light Locker Settings application class.'''

    def __init__(self):
        '''Initialize the Light Locker Settings application.'''
        self.systemwide_desktop_file = "/etc/xdg/autostart/light-locker.desktop"
        self.user_desktop_file = os.path.join(os.path.expanduser('~'),
                                              '.config',
                                              'autostart',
                                              'light-locker.desktop')
        self.screenblank_timeout_desktop_file = os.path.join(
            os.path.expanduser('~'),
            '.config',
            'autostart',
            'screensaver-settings.desktop')

        self.builder = Gtk.Builder()

        script_dir = os.path.dirname(os.path.abspath(__file__))
        glade_file = os.path.join(script_dir, "light-locker-settings.glade")
        self.builder.add_from_file(glade_file)
        self.builder.connect_signals(self)

        self.window = self.builder.get_object("light_locker_settings_window")
        self.window.set_title("Light Locker Settings")
        self.window.show()

        ''' Set background-color of frame to base-color to make it resemble the
        XfceHeading widget '''
        self.xfce_header = self.builder.get_object("xfce_header")
        style = self.xfce_header.get_style_context()
        base_color = style.lookup_color("theme_base_color")
        self.xfce_header.override_background_color(0, base_color[1])

        self.use_lightlocker = self.builder.get_object("use_lightlocker")
        self.lock_after_screenblank = self.builder.get_object(
            "lock_after_screenblank")
        self.infobar = self.builder.get_object("infobar1")
        self.infobar.hide()

        ''' Set up scales '''
        self.screenblank_timeout = self.builder.get_object(
            "screenblank_timeout")
        self.screenblank_timeout.add_mark(1, 3, None)
        self.screenoff_timeout = self.builder.get_object("screenoff_timeout")
        self.screenoff_timeout.add_mark(1, 3, None)
        self.lock_delay = self.builder.get_object("lock_delay")
        self.lock_delay.add_mark(0, 3, None)
        for i in range(1, 13):
            self.screenblank_timeout.add_mark(i * 10, 3, None)
            self.screenoff_timeout.add_mark(i * 10, 3, None)
            self.lock_delay.add_mark(i * 10, 3, None)

        ''' if there is a file exists in ~/.config/autostart/ parse that
            for current settings picked by the user else load in the
            systemwide '''
        if os.path.isfile(self.user_desktop_file):
            self.read_in_desktop(self.user_desktop_file)
        else:
            self.read_in_desktop(self.systemwide_desktop_file)

        ''' read in the X11 screensaver settings from bash '''
        screensaver_output = self.run_command('xset q')
        screenblank_timeout_grep = re.search(
            "timeout: *(\d+)", screensaver_output)
        if screenblank_timeout_grep:
            screenblank_timeout = re.findall(
                '\d+', screenblank_timeout_grep.group(1))
            self.screenblank_timeout.set_value(
                int(screenblank_timeout[0]) / 60)
        screenoff_timeout_grep = re.search(
            "Standby: *(\d+)", screensaver_output)
        if screenoff_timeout_grep:
            screenoff_timeout = re.findall(
                '\d+', screenoff_timeout_grep.group(1))
            self.screenoff_timeout.set_value(int(screenoff_timeout[0]) / 60)

        ''' Monitor changes to the settings '''
        self.apply = self.builder.get_object("apply")
        self.apply.set_sensitive(False)
        self.locksettings_changed = False
        self.screenblank_timeout.connect(
            "value-changed", self.screenblank_value_changed_cb)
        self.screenoff_timeout.connect(
            "value-changed", self.screenoff_value_changed_cb)
        self.lock_delay.connect(
            "value-changed", self.lock_delay_value_changed_cb)

    def run_command(self, cmd):
        '''Run a shell command, return its output.'''
        output = subprocess.check_output(cmd, shell=True)
        return output

    def read_in_desktop(self, filepath):
        '''Parse the .desktop file for light-locker settings.'''
        with open(filepath) as f:
            self.content = f.readlines()
            for line in self.content:
                if line.startswith("Exec="):
                    self.parse_options(line)

    def parse_options(self, options_line):
        '''Parse command-line arguments from the Exec line.'''
        if "light-locker" in options_line:
            self.use_lightlocker.set_active(True)

            ''' replace the = so we can parse it easily '''
            options_line = options_line.replace("=", " ")
            splitArgs = shlex.split(options_line)

            parser = argparse.ArgumentParser(
                description='Process the timeout until lock.')
            parser.add_argument("--lock-after-screensaver")
            (args, others) = parser.parse_known_args(splitArgs)

            if args.lock_after_screensaver:
                if int(args.lock_after_screensaver) != -1:
                    self.lock_after_screenblank.set_active(True)
                    lock_time_unscaled = self.light_locker_time_down_scaler(
                        int(args.lock_after_screensaver))
                    self.lock_delay.set_value(lock_time_unscaled)
                else:
                    self.lock_delay.set_sensitive(False)
            else:
                self.lock_after_screenblank.set_active(False)
                self.lock_delay.set_sensitive(False)
        else:
            self.lock_after_screenblank.set_active(False)
            self.lock_delay.set_sensitive(False)

    def write_light_locker_file(self, filepath, args_for_file):
        '''Write the updated light-locker settings to its .desktop file.'''
        with open(filepath, 'w') as f:
            for line in self.content:
                if line.startswith("Exec="):
                    f.write(args_for_file)
                else:
                    f.write(line)

    def write_screensaver_file(self, filepath, args_for_file):
        '''Write the updated screensaver settings to its .desktop file.'''
        with open(filepath, 'w') as f:
            f.write(
                "[Desktop Entry]\n"
                "Name=Screensaver\n"
                "Comment=Set screensaver timeouts\n")
            f.write(args_for_file)

    def use_lightlocker_cb(self, switch, gparam):
        '''Update the displayed lock controls when light-locker is enabled or
        disabled.'''
        ''' if on then allow for the timeout to be set '''
        self.locksettings_changed = True
        self.apply.set_sensitive(True)
        if switch.get_active():
            self.lock_delay.set_sensitive(False)
            self.lock_after_screenblank.set_sensitive(False)
        else:
            self.lock_after_screenblank.set_sensitive(True)
            if self.lock_after_screenblank.get_active():
                self.lock_delay.set_sensitive(True)

    def lock_after_screenblank_cb(self, switch, gparam):
        '''Update the displayed screen blanking controls when locking is
        enabled or disabled.'''
        ''' if on then allow for the timeout to be set '''
        self.locksettings_changed = True
        self.apply.set_sensitive(True)
        if switch.get_active():
            self.lock_delay.set_sensitive(False)
        else:
            self.lock_delay.set_sensitive(True)

    def screenblank_value_changed_cb(self, gparam):
        '''Sync screenblank and screenoff settings when values are modified.'''
        self.apply.set_sensitive(True)

        blank_timeout = int(self.screenblank_timeout.get_value())
        off_timeout = int(self.screenoff_timeout.get_value())

        ''' screenoff can never be shorter than screenblank-timeout '''
        if (blank_timeout >= off_timeout) and (off_timeout != 0):
            self.screenoff_timeout.set_value(blank_timeout)

    def screenoff_value_changed_cb(self, gparam):
        '''Sync screenblank and screenoff settings when values are modified.'''
        self.apply.set_sensitive(True)

        blank_timeout = int(self.screenblank_timeout.get_value())
        off_timeout = int(self.screenoff_timeout.get_value())

        ''' screenoff can never be shorter than screenblank-timeout '''
        if off_timeout <= blank_timeout:
            self.screenblank_timeout.set_value(off_timeout)

    def lock_delay_value_changed_cb(self, gparam):
        '''Enable saving of lock setting when the delay has been modified.'''
        self.locksettings_changed = True
        self.apply.set_sensitive(True)

    def screensaver_label_formatter(self, screenblank_timeout, max_value):
        '''Convert timeout values to a more friendly format.'''
        value = str(int(screenblank_timeout.get_value()))
        if value == "0":
            formatted_string = "Never"
        elif value == "1":
            formatted_string = "1 minute"
        else:
            formatted_string = value + " minutes"

        return formatted_string

    def light_locker_label_formatter(self, light_locker_slider, max_value):
        '''Convert timeout values to a more friendly format.'''
        value = str(int(light_locker_slider.get_value()))
        if value == "-1":
            formatted_string = "Never"
        elif value == "0":
            formatted_string = "Immediately"
        else:
            value = self.light_locker_time_up_scaler(
                int(light_locker_slider.get_value()))
            formatted_string = self.secs_to_readable(value)
        return formatted_string

    def light_locker_time_up_scaler(self, time):
        '''Scale times up.'''
        if time > 60:
            time = (time - 60) * 60
        return time

    def light_locker_time_down_scaler(self, time):
        '''Scale times down.'''
        if time > 60:
            time = time / 60 + 60
        return time

    def apply_cb(self, button, data=None):
        '''Apply changes and update the relevant setting files.'''
        ''' Check whether the autostart-directory exists and if not
        create it '''
        autostart_dir = os.path.dirname(self.user_desktop_file)
        if not os.path.exists(autostart_dir):
            os.makedirs(autostart_dir)

        light_locker_exec = "Exec="
        lock_delay_scaled = self.light_locker_time_up_scaler(
            int(self.lock_delay.get_value()))
        timeout = str(lock_delay_scaled)
        if not self.lock_after_screenblank.get_active():
            timeout = "-1"
        if self.use_lightlocker.get_active():
            light_locker_exec = light_locker_exec + \
                "light-locker --lock-after-screensaver=" + timeout + "\n"
        ''' save the light-locker settings '''
        self.write_light_locker_file(self.user_desktop_file, light_locker_exec)

        screenblank_timeout = str(
            int(self.screenblank_timeout.get_value()) * 60)
        screenoff_timeout = str(int(self.screenoff_timeout.get_value()) * 60)
        self.run_command('xset s ' + screenblank_timeout +
                           " dpms " + screenoff_timeout + " 0 0")
        x_exec = "Exec=xset s " + screenblank_timeout + \
            " dpms " + screenoff_timeout + " 0 0"
        ''' write any changes to the .desktop file '''
        self.write_screensaver_file(
            self.screenblank_timeout_desktop_file, x_exec)

        ''' Only show the infobar if light-locker settings have changed '''
        if self.locksettings_changed:
            self.infobar.show()

        self.run_command('xset s ' + screenblank_timeout)
        self.apply.set_sensitive(False)

    def secs_to_readable(self, seconds):
        '''Convert seconds to a more friendly format.'''
        mins = seconds / 60
        seconds -= 60 * mins

        if mins == 0:
            formatted_string = str(seconds) + " seconds"
        elif mins == 1 and seconds == 0:
            formatted_string = ("%d minute" % (mins))
        elif mins == 1:
            formatted_string = ("%d minute %d seconds" % (mins, seconds))
        elif mins > 1 and seconds == 0:
            formatted_string = ("%d minutes" % (mins))
        elif mins > 1 and seconds != 0:
            formatted_string = ("%d minutes %d seconds" % (mins, seconds))

        return formatted_string

    def on_window_destroy(self, *args):
        '''Exit the application when the window is closed.'''
        Gtk.main_quit()

    def on_close_clicked(self, *args):
        '''Exit the application when the window is closed.'''
        Gtk.main_quit()


if __name__ == "__main__":
    main = LightLockerSettings()
    Gtk.main()
